@@ -28,7 +28,7 @@ namespace FSpot {
 
 		// Limit pass control back to the main loop after
 		// chunk_timeout miliseconds.
-		int  chunk_timeout = 1000;
+		int  chunk_timeout = 100;
 
 		Delay delay;
 
@@ -77,7 +77,8 @@ namespace FSpot {
 			loader.AreaPrepared += HandleAreaPrepared;
 			loader.AreaUpdated += HandleAreaUpdated;
 			loader.Closed += HandleClosed;
-			
+
+			ThumbnailGenerator.Default.PushBlock ();
 			delay.Start ();
 		}			
 
@@ -210,6 +211,8 @@ namespace FSpot {
 
 			if (Done != null)
 				Done (this, System.EventArgs.Empty);
+			
+			ThumbnailGenerator.Default.PopBlock ();
 		}
 
 		public void Dispose ()
