@@ -19,14 +19,16 @@ public class FileImportBackend : ImportBackend {
 	{
 		if (path.ToLower().EndsWith (".jpg") || path.ToLower().EndsWith (".jpeg"))
 			file_paths.Add (path);
-
 	}
-	
+
 	private void GetListing (System.IO.DirectoryInfo info)
 	{
-		System.Console.WriteLine ("Scanning {0}", info.FullName);
+		GetListing (info, info.GetFiles (), recurse);
+	}
 
-		System.IO.FileInfo [] files = info.GetFiles ();
+	private void GetListing (System.IO.DirectoryInfo info, System.IO.FileInfo [] files, bool recurse)
+	{
+		System.Console.WriteLine ("Scanning {0}", info.FullName);
 		Hashtable exiting_entries = new Hashtable ();
 
 		foreach (Photo p in store.Query (info)) {
@@ -64,18 +66,16 @@ public class FileImportBackend : ImportBackend {
 
 		foreach (string path in base_paths) {
 			try {	
-				GetListing (new System.IO.DirectoryInfo (path));
+				if (System.IO.Directory.Exists (path))
+					GetListing (new System.IO.DirectoryInfo (path));
+				else if (System.IO.File.Exists (path))
+					GetListing (new System.IO.DirectoryInfo (System.IO.Path.GetDirectoryName (path)), 
+						    new System.IO.FileInfo [] { new System.IO.FileInfo (path)}, false);
 			} catch (Exception e) {
 				System.Console.WriteLine (e.ToString ());
-				AddPath (path);
 			}
 		}	
 
-		if (file_paths.Count == 0) {
-			file_paths = null;
-			return 0;
-		}
-
 		imported_photos = new ArrayList ();
 
 		return file_paths.Count;
@@ -128,6 +128,7 @@ public class FileImportBackend : ImportBackend {
 		foreach (Photo p in imported_photos) {
 			FSpot.ThumbnailGenerator.Default.Request (p.DefaultVersionPath, 0, 256, 256);
 		}
+
 		imported_photos = null;
 		count = 0;
 	}
