@@ -53,6 +53,12 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
+	public FSpot.PixbufCache Cache {
+		get {
+			return cache;
+		}
+	}
+
 	private bool display_tags = true;
 	public bool DisplayTags {
 		get {
@@ -104,6 +110,7 @@ public class IconView : Gtk.Layout {
 	// scroll region).  Used to compute the area that went offscreen in the "changed"
 	// signal handler for the vertical GtkAdjustment.
 	private int y_offset;
+	private int x_offset;
 
 	// Hash of all the order number of the items that are selected.
 	private Hashtable selected_cells;
@@ -447,10 +454,7 @@ public class IconView : Gtk.Layout {
 			return;
 		
 		FSpot.IBrowsableItem photo = collection.Items [thumbnail_num];
-		string thumbnail_path = Thumbnail.PathForUri (photo.DefaultVersionUri.ToString (), 
-							      ThumbnailSize.Large);
-
-		
+		string thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 		
 		FSpot.PixbufCache.CacheEntry entry = cache.Lookup (thumbnail_path);
 		if (entry == null)
@@ -458,16 +462,9 @@ public class IconView : Gtk.Layout {
 		else
 			entry.Data = thumbnail_num;
 
-			
-		Gdk.GC gc = new Gdk.GC (BinWindow);
-		gc.Copy (Style.ForegroundGC (StateType.Normal));
-		gc.SetLineAttributes (1, LineStyle.Solid, CapStyle.NotLast, JoinStyle.Round);
 		bool selected = CellIsSelected (thumbnail_num);
-		
-
 		StateType cell_state = selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal;
 		
-		
 		Style.PaintFlatBox (Style, BinWindow, cell_state, 
 				    ShadowType.Out, area, this, "IconView", 
 				    bounds.X, bounds.Y,
@@ -510,13 +507,13 @@ public class IconView : Gtk.Layout {
 										region.Width, region.Height);
 					
 					if (entry.Reload && expansion == 0)
-						cache.Update (entry, PixbufUtils.ShallowCopy (thumbnail));
-
+						cache.Update (entry, PixbufUtils.ShallowCopy (temp_thumbnail));
+					
 				} else {
 					temp_thumbnail = thumbnail.ScaleSimple (region.Width, region.Height, 
 										InterpType.Bilinear);
 				}
-				
+
 				PixbufUtils.CopyThumbnailOptions (thumbnail, temp_thumbnail);
 			} else
 				temp_thumbnail = thumbnail;
@@ -548,9 +545,11 @@ public class IconView : Gtk.Layout {
 			if (temp_thumbnail != thumbnail)
 				temp_thumbnail.Dispose ();
 			
-			thumbnail.Dispose ();
 		}
-			
+		
+		if (thumbnail != null) 
+			thumbnail.Dispose ();
+		
 		Gdk.Rectangle layout_bounds = Gdk.Rectangle.Zero;
 		if (DisplayDates) {
 			string date;
@@ -631,6 +630,9 @@ public class IconView : Gtk.Layout {
 
 	private void DrawAllCells (Gdk.Rectangle area)
 	{
+		if (cell_width == 0 || cell_height == 0)
+			return;
+			
 		int start_cell_column = Math.Max ((area.X - BORDER_SIZE) / cell_width, 0);
 		int start_cell_row = Math.Max ((area.Y - BORDER_SIZE) / cell_height, 0);
 		int start_cell_num = start_cell_column + start_cell_row * cells_per_row;
@@ -646,34 +648,7 @@ public class IconView : Gtk.Layout {
 					 cells_per_row - start_cell_column);
 
 		int i, cell_num;
-
-		FSpot.IBrowsableItem photo;
-		FSpot.PixbufCache.CacheEntry entry;
-		string thumbnail_path;
-
-		
-		// Preload the cache with images aroud the expose area
-		// FIXME the preload need to be tuned to the Cache size but this is a resonable start
-		
-		int len = (end_cell_row - start_cell_row + 20) * cells_per_row;
-		int scell = System.Math.Max ((start_cell_row - 10) * cells_per_row, 0);
-		int ecell = scell + len;
-		if (scell > collection.Items.Length - len) {
-		        ecell = collection.Items.Length;
-			scell = System.Math.Max (0, scell - len);
-		} else
-			ecell = scell + len;
-			
-		for (i = scell; i < ecell; i++) {
-			photo = collection.Items [i];
-			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
-
-			entry = cache.Lookup (thumbnail_path);
-			if (entry == null)
-				cache.Request (thumbnail_path, i, ThumbnailWidth, ThumbnailHeight);
-				
-		}
-			
+		//Preload (area, false);
 
 		for (i = 0, cell_num = start_cell_num;
 		     i < num_rows && cell_num < collection.Items.Length;
@@ -692,7 +667,6 @@ public class IconView : Gtk.Layout {
 
 	}
 
-
 	private void GetCellPosition (int cell_num, out int x, out int y)
 	{
 		if (cells_per_row == 0) {
@@ -711,40 +685,93 @@ public class IconView : Gtk.Layout {
 
 	// Scrolling.  We do this in an idle loop so we can catch up if the user scrolls quickly.
 
-	private uint scroll_on_idle_id;
-
-	private int idle_count;	// FIXME
-
-	private bool HandleScrollOnIdle ()
+	private void Scroll ()
 	{
-		Adjustment adjustment = Vadjustment;
+		int ystep = (int)(Vadjustment.Value - y_offset);
+		int xstep = (int)(Hadjustment.Value - x_offset);
+		
+		if (xstep > 0)
+			xstep = Math.Max (xstep, Allocation.Width);
+		else 
+			xstep = Math.Min (xstep, -Allocation.Width);
 
-		if (y_offset == adjustment.Value)
-			return false;
+		if (ystep > 0)
+			ystep = Math.Max (ystep, Allocation.Height);
+		else 
+			ystep = Math.Min (ystep, -Allocation.Height);
 
-		y_offset = (int) adjustment.Value;
-		scroll_on_idle_id = 0;
-		return false;
+		Gdk.Rectangle area;
+
+		Gdk.Region offscreen = new Gdk.Region ();
+		/*
+		System.Console.WriteLine ("step ({0}, {1}) allocation ({2},{3},{4},{5})", 
+					  xstep, ystep, Hadjustment.Value, Vadjustment.Value, 
+					  Allocation.Width, Allocation.Height);
+		*/
+		/*		
+		area = new Gdk.Rectangle (Math.Max ((int) (Hadjustment.Value + 4 * xstep), 0),  
+					  Math.Max ((int) (Vadjustment.Value + 4 * ystep), 0),
+					  Allocation.Width,
+					  Allocation.Height);
+		offscreen.UnionWithRect (area);
+		area = new Gdk.Rectangle (Math.Max ((int) (Hadjustment.Value + 3 * xstep), 0),  
+					  Math.Max ((int) (Vadjustment.Value + 3 * ystep), 0),
+					  Allocation.Width,
+					  Allocation.Height);
+		offscreen.UnionWithRect (area);
+		*/
+		area = new Gdk.Rectangle (Math.Max ((int) (Hadjustment.Value + 2 * xstep), 0),  
+					  Math.Max ((int) (Vadjustment.Value + 2 * ystep), 0),
+					  Allocation.Width,
+					  Allocation.Height);
+		offscreen.UnionWithRect (area);
+		area = new Gdk.Rectangle (Math.Max ((int) (Hadjustment.Value + xstep), 0),  
+					  Math.Max ((int) (Vadjustment.Value + ystep), 0),
+					  Allocation.Width,
+					  Allocation.Height);		
+		offscreen.UnionWithRect (area);
+		area = new Gdk.Rectangle ((int) Hadjustment.Value, 
+					  (int) Vadjustment.Value,
+					  Allocation.Width,
+					  Allocation.Height);
+
+		// always load the onscreen area last to make sure it
+		// is first in the loading
+		Gdk.Region onscreen = Gdk.Region.Rectangle (area);
+		offscreen.Subtract (onscreen);
+
+		PreloadRegion (offscreen, ystep);
+		offscreen.Destroy ();
+		//PreloadRegion (onscreen, ystep);
+		Preload (area, false);
+		onscreen.Destroy ();
+
+		y_offset = (int) Vadjustment.Value;
+		x_offset = (int) Hadjustment.Value;
+	}
+	
+        private void PreloadRegion (Gdk.Region region, int step)
+	{
+		Gdk.Rectangle [] rects = region.GetRectangles ();
+		
+		if (step < 0)
+			System.Array.Reverse (rects);
+
+		foreach (Gdk.Rectangle preload in rects) {
+			Preload (preload, false);
+		}
 	}
 
-	private void Scroll ()
+	private void Preload (Gdk.Rectangle area, bool back)
 	{
 		if (cells_per_row ==0)
 			return;
 
-		Gdk.Rectangle area = new Gdk.Rectangle ((int) Hadjustment.Value, 
-							(int) Vadjustment.Value,
-							Allocation.Width,
-							Allocation.Height);
-
 		int start_cell_column = Math.Max ((area.X - BORDER_SIZE) / cell_width, 0);
 		int start_cell_row = Math.Max ((area.Y - BORDER_SIZE) / cell_height, 0);
 		int start_cell_num = start_cell_column + start_cell_row * cells_per_row;
 
-		int start_cell_x, cell_y;
-		GetCellPosition (start_cell_num, out start_cell_x, out cell_y);
-
-		//int end_cell_column = Math.Max ((area.X + area.Width - BORDER_SIZE) / cell_width, 0);
+		int end_cell_column = Math.Max ((area.X + area.Width - BORDER_SIZE) / cell_width, 0);
 		int end_cell_row = Math.Max ((area.Y + area.Height - BORDER_SIZE) / cell_height, 0);
 
 		int i;
@@ -756,36 +783,39 @@ public class IconView : Gtk.Layout {
 		// Preload the cache with images aroud the expose area
 		// FIXME the preload need to be tuned to the Cache size but this is a resonable start
 		
-		int len = (end_cell_row - start_cell_row + 10) * cells_per_row;
-		int scell = System.Math.Max ((start_cell_row - 5) * cells_per_row, 0);
+		int cols = end_cell_column - start_cell_column + 1;
+		int rows = end_cell_row - start_cell_row + 1;
+		int len = rows * cols;
+		int scell = start_cell_num;
 		int ecell = scell + len;
 		if (scell > collection.Items.Length - len) {
 		        ecell = collection.Items.Length;
 			scell = System.Math.Max (0, scell - len);
 		} else
 			ecell = scell + len;
-			
-		for (i = scell; i < ecell; i++) {
-			photo = collection.Items [i];
-			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
+		
+		int mid = (ecell - scell) / 2;
+		for (i = 0; i < mid; i++)
+		{
+			int cell = back ? ecell - i - 1 : scell + mid + i;
 
+			photo = collection.Items [cell];
+			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
+			
 			entry = cache.Lookup (thumbnail_path);
 			if (entry == null)
-				cache.Request (thumbnail_path, i, ThumbnailWidth, ThumbnailHeight);
-				
-		}
+				cache.Request (thumbnail_path, cell, ThumbnailWidth, ThumbnailHeight);
+
+			cell = back ? scell + i : scell + mid - i - 1;
+			photo = collection.Items [cell];
+			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 			
-		if (scroll_on_idle_id == 0)
-			scroll_on_idle_id = GLib.Idle.Add (new GLib.IdleHandler (HandleScrollOnIdle));
+			entry = cache.Lookup (thumbnail_path);
+			if (entry == null)
+				cache.Request (thumbnail_path, cell, ThumbnailWidth, ThumbnailHeight);
+		} 
 	}
 
-	private void CancelScroll ()
-	{
-		if (scroll_on_idle_id != 0)
-			GLib.Source.Remove (scroll_on_idle_id);
-	}
-
-
 	//
 	// The throb interface
 	//
@@ -840,11 +870,7 @@ public class IconView : Gtk.Layout {
 		else if (y + cell_height > adjustment.Value + adjustment.PageSize)
 			adjustment.Value = y + cell_height - adjustment.PageSize;
 		
-#if true
-		Scroll ();
-#else
 		adjustment.Change ();
-#endif		
 	}
 
 
@@ -1079,7 +1105,6 @@ public class IconView : Gtk.Layout {
 
 	private void HandleDestroyEvent (object sender, DestroyEventArgs args)
 	{
-		CancelScroll ();
 		CancelThrob ();
 	}
 }
