@@ -1,8 +1,6 @@
 namespace FSpot {
 	public class ThumbnailGenerator : PixbufLoader {
-
 		private static Gnome.ThumbnailFactory factory = new Gnome.ThumbnailFactory (Gnome.ThumbnailSize.Large);
-
 		static public ThumbnailGenerator Default = new ThumbnailGenerator ();
 
 		public static Gdk.Pixbuf Create (string path)
@@ -16,6 +14,21 @@ namespace FSpot {
 			}
 		}
 		
+		public static bool ThumbnailIsValid (Gdk.Pixbuf thumbnail, System.Uri uri)
+		{
+			bool valid = false;
+
+			try {			
+				System.DateTime mtime = System.IO.File.GetLastWriteTime (uri.LocalPath);
+				valid  = Gnome.Thumbnail.IsValid (thumbnail, uri.ToString (), mtime);
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e);
+				valid = false;
+			}
+			
+			return valid;
+		}
+
 		public static string ThumbnailPath (System.Uri uri)
 		{
 			string large_path = Gnome.Thumbnail.PathForUri (uri.ToString (), Gnome.ThumbnailSize.Large);
@@ -40,15 +53,23 @@ namespace FSpot {
 			//			  image.GetOption ("tEXt::Thumb::URI"), 
 			//			  image.GetOption ("tEXt::Thumb::MTime"));
 			
-			string large_path = Gnome.Thumbnail.PathForUri (uri, Gnome.ThumbnailSize.Large);
+			string large_path = ThumbnailPath (uri);
 			try {
 				ThumbnailCache.Default.RemoveThumbnailForPath (large_path);
 			} finally {
 				factory.SaveThumbnail (image, uri, mtime);
 			}
+		}
 
-			if (!PhotoLoader.ThumbnailIsValid (new System.Uri (uri), image))
-			    System.Console.WriteLine ("************************************ Argh");
+		protected override void EmitLoaded (System.Collections.Queue results)
+		{
+			base.EmitLoaded (results);
+			
+			foreach (RequestItem r in results) {
+				if (r.result != null)
+					r.result.Dispose ();
+			}
+				
 		}
 
 		protected override void ProcessRequest (RequestItem request)
@@ -58,10 +79,7 @@ namespace FSpot {
 			Gdk.Pixbuf image = request.result;
 			if (image != null) {
 				Save (image, request.path);
-				//image.Dispose ();
 			}
-
-			//System.Threading.Thread.Sleep (50);
 		}
 	}
 }
